-- Static.
local ICON_SIZE = 14
local ICON_OFFSET = 7; -- Icon is 28x28
local MapId = 1033;

--local upperDungeonMapLevel = 32	-- debug / non-instance
local upperDungeonMapLevel = 41 -- instance

--local lowerDungeonMapLevel = 33	-- debug / non-instance
local lowerDungeonMapLevel = 42 -- instance

Withered = {
	hasSetup = false,
	icons = {}, -- Used to store the icons we cook!
};

local w = Withered;

-- Event frame! How.. eventful.
local eventFrame = CreateFrame("FRAME");
eventFrame:RegisterEvent("WORLD_MAP_UPDATE");
eventFrame:SetScript("OnEvent", function(...) w.OnEvent(...); end);

w.OnEvent = function(self, event, ...)

	--DEFAULT_CHAT_FRAME:AddMessage("GetCurrentMapAreaID:  " .. GetCurrentMapAreaID())
	
	-- The world map has updated, check our stuff!
	if event == "WORLD_MAP_UPDATE" then
		-- Check the world map is actually shown.
		if WorldMapFrame:IsShown() then
			-- Confirm that we're in the correct zone (and multi-map level).
			local currentMapDungeonLevel = GetCurrentMapDungeonLevel();
			if GetCurrentMapAreaID() == MapId and 
				( currentMapDungeonLevel == upperDungeonMapLevel or currentMapDungeonLevel == lowerDungeonMapLevel) then
				w.UpdateIcons(currentMapDungeonLevel);
				return;
			end
		end
		
		-- Negative, commander! Hide the icons.
		w.HideIcons();
	end
end

w.UpdateIcons = function(currentMapDungeonLevel)
	-- Set-up icons for the first time.
	if not w.hasSetup then
		w.Setup(TreasureChests.Details);
	end

	--Khoria_DebugTools:PrintTable(TreasureChests.Details)
	--Khoria_DebugTools:PrintTable(w.icons)

	-- Iterate every spawned icon and update it.
	local frameWidth, frameHeight = WorldMapPOIFrame:GetSize();
	for key, icon in pairs(w.icons) do
		-- set the icons when WorldMap is updating (eg. Zoom)
		if (not w.IsCoordinateOnMapLevel(currentMapDungeonLevel, icon.mapLevel)) then
			icon:Hide();
		else
			icon:SetPoint("TOPLEFT", (frameWidth * icon.Xpos) - ICON_OFFSET, (frameHeight * icon.Ypos) + ICON_OFFSET);
			icon:Show();
		end
	end
end

w.IsCoordinateOnMapLevel = function(currentMapDungeonLevel, iconMapLevel)
	return  
		iconMapLevel == currentMapDungeonLevel
end

w.HideIcons = function()
	-- Iterate every icon and hide it.
	for key, icon in pairs(w.icons) do
		icon:Hide();
	end
end

w.UpdateIconState = function(icon)
	-- check if Quest is completed and change texture if needed

	icon.isActive = true;

	if IsQuestFlaggedCompleted(icon.questId) then
		icon.texture:SetTexture(icon.icon_file);
		icon.texture:SetDesaturated(nil); 
	else
		icon.texture:SetTexture(icon.icon_file);
		icon.texture:SetDesaturated(1);
	end
end

w.OnIconUpdate = function(self, elapsed)
	-- Every second, check our icons are in the right place.
	-- Lots of things can displace them, but we don't need to go over-board on checking.
	if self.updateTimer >= 1 then
		-- Get the current width/height of the POI frame.
		local frameWidth, frameHeight = WorldMapPOIFrame:GetSize();
		self:SetFrameStrata("TOOLTIP"); -- Map frame resets strata, so we enforce it here every time.
		self:SetPoint("TOPLEFT", (frameWidth * self.Xpos) - ICON_OFFSET, (frameHeight * self.Ypos) + ICON_OFFSET);

		w.UpdateIconState(self);

		self.updateTimer = 0;
	else
		self.updateTimer = self.updateTimer + elapsed;
	end
end

w.OnIconEnter = function(self)

	local description = self.name .. "\r\n" .. self.description;

	if IsQuestFlaggedCompleted(self.questId) then
		description = description .. " (found) "
	end

	WorldMapFrameAreaLabel:SetText("Withered Army Training Map");	
	WorldMapFrameAreaDescription:SetText(description);

	WorldMapFrameAreaLabel:Show();
	WorldMapFrameAreaDescription:Show();

end

w.OnIconLeave = function(self)
	WorldMapFrameAreaLabel:SetText("");
	WorldMapFrameAreaDescription:SetText("");
end

w.Setup = function(itemInformation)

	-- Create a re-usable template for frame creation.
	local template = {
		size = ICON_SIZE,
		parent = WorldMapPOIFrame,
		strata = "TOOLTIP",
		textures = {
			injectSelf = "texture",
			texture = INACTIVE_ICON,
		},
		scripts = {
			OnUpdate = w.OnIconUpdate,
			OnEnter = w.OnIconEnter,
			OnLeave = w.OnIconLeave
		}
	};

	-- Create a new frame for every point.
	for key, item in pairs(itemInformation) do
		-- Convert the generic point to an actual offset for anchoring.
		item.Xpos = item.Xpos / 100;
		item.Ypos = -(item.Ypos / 100);
				
		template.data = item; -- Provide point data to the frame.
		template.data.updateTimer = 0; -- Used in OnIconUpdate

		local frame = Krutilities:Frame(template); -- Create our new icon frame.
		table.insert(w.icons, frame); -- Store the frame in our table for later.

		w.UpdateIconState(frame); -- Initial state update.
	end

	w.hasSetup = true; -- Flag set-up as done.
end
